<?php

namespace MollieForms;


class Admin {

    private $db, $mollieForms, $helpers;

    /**
     * Admin constructor.
     * @param MollieForms $plugin
     */
    public function __construct($plugin)
    {
        global $wpdb;

        $this->db = $wpdb;
        $this->mollieForms  = $plugin;
        $this->helpers      = new Helpers();
        $this->initAdmin();
    }

    /**
     * Init admin
     */
    private function initAdmin()
    {
        add_action('init', function() {
            remove_post_type_support('mollie-forms', 'editor');
        });
        add_action('admin_menu', array($this, 'adminMenu'));
        add_action('admin_enqueue_scripts', array($this, 'loadScripts'));
        add_action('add_meta_boxes_mollie-forms', array($this, 'addMetaBoxes'));
        add_action('save_post_mollie-forms', array($this, 'saveMetaBoxes'), 10, 2);
        add_action('admin_post_mollie-forms_export', array($this, 'exportRegistrations'));

        add_filter('plugin_row_meta', array($this, 'pluginRowMeta'), 10, 2);
        add_filter('post_row_actions', array($this, 'postActions'), 10, 2);
    }

    /**
     * Add items to Wordpress menu
     */
    public function adminMenu()
    {
        // Registrations
        add_submenu_page(
            'edit.php?post_type=mollie-forms',
            __('Registrations', 'mollie-forms'),
            __('Registrations', 'mollie-forms'),
            'edit_posts',
            'registrations',
            array(
                $this,
                'pageRegistrations'
            )
        );

        // Single registration
        add_submenu_page(
            null,
            __('Registration', 'mollie-forms'),
            __('Registration', 'mollie-forms'),
            'edit_posts',
            'registration',
            array(
                $this,
                'pageRegistration'
            )
        );

        // Add-ons
        add_submenu_page(
            'edit.php?post_type=mollie-forms',
            __('Add-ons', 'mollie-forms'),
            __('Add-ons', 'mollie-forms'),
            'administrator',
            'add-ons',
            array(
                $this,
                'pageAddons'
            )
        );

        // Support
        global $submenu;
        $submenu['edit.php?post_type=mollie-forms'][] = array('Support', 'manage_options', 'https://support.wobbie.nl');
    }

    /**
     * Add links to plugin meta
     *
     * @param $links
     * @param $file
     * @return array
     */
    public function pluginRowMeta($links, $file)
    {
        if ($this->mollieForms->getBaseName() == $file)
        {
            $row_meta = array(
                'support'    => '<a href="https://support.wobbie.nl" target="_blank">' . esc_html__('Support', 'mollie-forms') . '</a>',
                'add-ons'    => '<a href="edit.php?post_type=mollie-forms&page=add-ons">' . esc_html__('Add-ons', 'mollie-forms') . '</a>',
            );

            return array_merge($links, $row_meta);
        }

        return (array) $links;
    }

    /**
     * Load JavaScripts and Stylesheets
     */
    public function loadScripts()
    {
        wp_enqueue_script('mollie-forms_admin_scripts', $this->mollieForms->getDirUrl() . 'includes/js/admin-scripts.js', array('jquery', 'jquery-ui-core', 'jquery-ui-sortable', 'jquery-ui-tabs'), $this->mollieForms->getVersion());
        wp_enqueue_style('mollie-forms_admin_styles', $this->mollieForms->getDirUrl() . 'includes/css/admin-styles.css', array(), $this->mollieForms->getVersion());

        wp_register_style('jQueryUI', 'https://code.jquery.com/ui/1.12.1/themes/base/jquery-ui.css');
        wp_enqueue_style('jQueryUI');
    }

    /**
     * Replace the actions of a Wordpress Post
     *
     * @param $actions
     * @param $post
     * @return mixed
     */
    public function postActions($actions, $post)
    {
        if ($post->post_type == 'mollie-forms')
        {
            unset($actions['inline hide-if-no-js']);
            unset($actions['view']);
            $actions['registrations'] = '<a href="edit.php?post_type=mollie-forms&page=registrations&post=' . $post->ID . '">' . __('Registrations', 'mollie-forms') . '</a>';
            $actions['export'] = '<a href="' . admin_url('admin-post.php?action=mollie-forms_export&post=' . $post->ID) . '">' . __('Export', 'mollie-forms') . '</a>';
        }
        return $actions;
    }

    /**
     * Register post meta boxes
     *
     * @param $post
     */
    public function addMetaBoxes($post)
    {
        add_meta_box('mollie-forms_fields', __('Fields', 'mollie-forms'), array($this, 'metaBoxFields'), 'mollie-forms', 'normal', 'high');
        add_meta_box('mollie-forms_settings', __('Settings', 'mollie-forms'), array($this, 'metaBoxSettings'), 'mollie-forms', 'normal', 'default');
        add_meta_box('mollie-forms_priceoptions', __('Price options', 'mollie-forms'), array($this, 'metaBoxPriceOptions'), 'mollie-forms', 'normal', 'default');
        add_meta_box('mollie-forms_emails', __('Email settings', 'mollie-forms'), array($this, 'metaBoxEmails'), 'mollie-forms', 'normal', 'default');
        add_meta_box('mollie-forms_paymentmethods', __('Payment methods', 'mollie-forms'), array($this, 'metaBoxPaymentMethods'), 'mollie-forms', 'side', 'default');
        remove_meta_box('slugdiv', 'mollie-forms', 'normal');
    }

    /**
     * Form fields meta box
     *
     * @param $post
     */
    public function metaBoxFields($post)
    {
        wp_nonce_field(basename(__FILE__), 'rfmp_meta_box_fields_nonce');
        $api_type       = get_post_meta($post->ID, '_rfmp_api_type', true) ?: 'payments';
        $field_type     = get_post_meta($post->ID, '_rfmp_fields_type', true);
        $field_label    = get_post_meta($post->ID, '_rfmp_fields_label', true);
        $field_value    = get_post_meta($post->ID, '_rfmp_fields_value', true);
        $field_class    = get_post_meta($post->ID, '_rfmp_fields_class', true);
        $field_required = get_post_meta($post->ID, '_rfmp_fields_required', true);

        if (empty($field_type)) {
            $field_type = array(0 => 'name', 1 => 'email', 2 => 'priceoptions', 3 => 'payment_methods', 4 => 'total', 5 => 'submit');
            $field_label = array(0 => __('Name', 'mollie-forms'), 1 => __('Email', 'mollie-forms'), 2 => '', 3 => __('Payment method', 'mollie-forms'), 5 => __('Submit', 'mollie-forms'));
        }

        include $this->mollieForms->getDirPath() . 'templates/metaboxes/fields.php';
    }

    /**
     * Form settings meta box
     *
     * @param $post
     */
    public function metaBoxSettings($post)
    {
        wp_nonce_field(basename(__FILE__), 'rfmp_meta_box_settings_nonce');
        $api_key            = get_post_meta($post->ID, '_rfmp_api_key', true);
        $api_type           = get_post_meta($post->ID, '_rfmp_api_type', true);
        $display_label      = get_post_meta($post->ID, '_rfmp_label_display', true);
        $display_pm         = get_post_meta($post->ID, '_rfmp_payment_methods_display', true);
        $display_po         = get_post_meta($post->ID, '_rfmp_priceoptions_display', true);
        $class_success      = get_post_meta($post->ID, '_rfmp_class_success', true);
        $class_error        = get_post_meta($post->ID, '_rfmp_class_error', true);
        $payment_description= get_post_meta($post->ID, '_rfmp_payment_description', true);
        $after_payment      = get_post_meta($post->ID, '_rfmp_after_payment', true);
        $message_success    = get_post_meta($post->ID, '_rfmp_msg_success', true);
        $message_error      = get_post_meta($post->ID, '_rfmp_msg_error', true);
        $redirect_success   = get_post_meta($post->ID, '_rfmp_redirect_success', true);
        $redirect_error     = get_post_meta($post->ID, '_rfmp_redirect_error', true);
        $class_form         = get_post_meta($post->ID, '_rfmp_class_form', true);
        $currency           = get_post_meta($post->ID, '_rfmp_currency', true);
        $locale             = get_post_meta($post->ID, '_rfmp_locale', true);
        $shippingCosts      = get_post_meta($post->ID, '_rfmp_shipping_costs', true);
        $vatSetting         = get_post_meta($post->ID, '_rfmp_vat_setting', true);

        include $this->mollieForms->getDirPath() . 'templates/metaboxes/settings.php';
    }

    /**
     * Price options meta box
     *
     * @param $post
     */
    public function metaBoxPriceOptions($post)
    {
        wp_nonce_field(basename(__FILE__), 'rfmp_meta_box_priceoptions_nonce');
        $priceOptions = $this->db->get_results("SELECT * FROM {$this->mollieForms->getPriceOptionsTable()} WHERE post_id=" . (int) $post->ID . " ORDER BY sort_order ASC");

        include $this->mollieForms->getDirPath() . 'templates/metaboxes/priceOptions.php';
    }

    /**
     * Emails meta box
     *
     * @param $post
     */
    public function metaBoxEmails($post)
    {
        wp_nonce_field(basename(__FILE__), 'rfmp_meta_box_emails_nonce');

        $enabled_paid_customer      = get_post_meta($post->ID, '_rfmp_enabled_paid_customer', true);
        $email_paid_customer        = get_post_meta($post->ID, '_rfmp_email_paid_customer', true);
        $subject_paid_customer      = get_post_meta($post->ID, '_rfmp_subject_paid_customer', true);
        $fromemail_paid_customer    = get_post_meta($post->ID, '_rfmp_fromemail_paid_customer', true);
        $fromname_paid_customer     = get_post_meta($post->ID, '_rfmp_fromname_paid_customer', true);

        $enabled_expired_customer      = get_post_meta($post->ID, '_rfmp_enabled_expired_customer', true);
        $email_expired_customer        = get_post_meta($post->ID, '_rfmp_email_expired_customer', true);
        $subject_expired_customer      = get_post_meta($post->ID, '_rfmp_subject_expired_customer', true);
        $fromemail_expired_customer    = get_post_meta($post->ID, '_rfmp_fromemail_expired_customer', true);
        $fromname_expired_customer     = get_post_meta($post->ID, '_rfmp_fromname_expired_customer', true);

        $enabled_cancelled_customer      = get_post_meta($post->ID, '_rfmp_enabled_cancelled_customer', true);
        $email_cancelled_customer        = get_post_meta($post->ID, '_rfmp_email_cancelled_customer', true);
        $subject_cancelled_customer      = get_post_meta($post->ID, '_rfmp_subject_cancelled_customer', true);
        $fromemail_cancelled_customer    = get_post_meta($post->ID, '_rfmp_fromemail_cancelled_customer', true);
        $fromname_cancelled_customer     = get_post_meta($post->ID, '_rfmp_fromname_cancelled_customer', true);

        $enabled_paid_merchant      = get_post_meta($post->ID, '_rfmp_enabled_paid_merchant', true);
        $email_paid_merchant        = get_post_meta($post->ID, '_rfmp_email_paid_merchant', true);
        $subject_paid_merchant      = get_post_meta($post->ID, '_rfmp_subject_paid_merchant', true);
        $fromemail_paid_merchant    = get_post_meta($post->ID, '_rfmp_fromemail_paid_merchant', true);
        $fromname_paid_merchant     = get_post_meta($post->ID, '_rfmp_fromname_paid_merchant', true);
        $toemail_paid_merchant      = get_post_meta($post->ID, '_rfmp_toemail_paid_merchant', true);

        $enabled_expired_merchant      = get_post_meta($post->ID, '_rfmp_enabled_expired_merchant', true);
        $email_expired_merchant        = get_post_meta($post->ID, '_rfmp_email_expired_merchant', true);
        $subject_expired_merchant      = get_post_meta($post->ID, '_rfmp_subject_expired_merchant', true);
        $fromemail_expired_merchant    = get_post_meta($post->ID, '_rfmp_fromemail_expired_merchant', true);
        $fromname_expired_merchant     = get_post_meta($post->ID, '_rfmp_fromname_expired_merchant', true);
        $toemail_expired_merchant      = get_post_meta($post->ID, '_rfmp_toemail_expired_merchant', true);

        $enabled_cancelled_merchant      = get_post_meta($post->ID, '_rfmp_enabled_cancelled_merchant', true);
        $email_cancelled_merchant        = get_post_meta($post->ID, '_rfmp_email_cancelled_merchant', true);
        $subject_cancelled_merchant      = get_post_meta($post->ID, '_rfmp_subject_cancelled_merchant', true);
        $fromemail_cancelled_merchant    = get_post_meta($post->ID, '_rfmp_fromemail_cancelled_merchant', true);
        $fromname_cancelled_merchant     = get_post_meta($post->ID, '_rfmp_fromname_cancelled_merchant', true);
        $toemail_cancelled_merchant      = get_post_meta($post->ID, '_rfmp_toemail_cancelled_merchant', true);

        $rfmp_editor_settings   = array();

        include $this->mollieForms->getDirPath() . 'templates/metaboxes/emails.php';
    }

    public function metaBoxPaymentMethods($post)
    {
        wp_nonce_field(basename(__FILE__), 'rfmp_meta_box_paymentmethods_nonce');
        $api_key    = get_post_meta($post->ID, '_rfmp_api_key', true);
        $api_type   = get_post_meta($post->ID, '_rfmp_api_type', true) ?: 'payments';
        $active     = get_post_meta($post->ID, '_rfmp_payment_method', true);
        $fixed      = get_post_meta($post->ID, '_rfmp_payment_method_fixed', true);
        $variable   = get_post_meta($post->ID, '_rfmp_payment_method_variable', true);
        $currency   = get_post_meta($post->ID, '_rfmp_currency', true) ?: 'EUR';
        $locale     = get_post_meta($post->ID, '_rfmp_locale', true) ?: null;

        if (!$locale && key_exists(get_locale(), $this->helpers->getLocales()))
            $locale = get_locale();

        try {

            if (!$api_key)
                echo '<p style="color: red">' . esc_html__('No API-key set', 'mollie-forms') . '</p>';
            else
            {
                $mollie = new MollieApi($api_key);

                // use billingCountry "DE" to show all methods
                $params = array('locale' => $locale, 'resource' => $api_type, 'billingCountry' => 'DE', 'includeWallets' => 'applepay');

                foreach ($mollie->all('methods', $params) as $method) {
                    echo '<input type="hidden" value="0" name="rfmp_payment_method[' . $method->id . ']">';
                    echo '<label><input type="checkbox" name="rfmp_payment_method[' . $method->id . ']" ' . (isset($active[$method->id]) && $active[$method->id] ? 'checked' : '') . ' value="1"> <img style="vertical-align:middle;display:inline-block;width:25px;" src="' . esc_url($method->image->svg) . '"> ' . esc_html($method->description) . '</label><br>';
                    echo esc_html_e('Surcharge:', 'mollie-forms') . ' ' . $this->helpers->getCurrencySymbol($currency ?: 'EUR') . ' <input type="number" step="any" min="0" name="rfmp_payment_method_fixed[' . $method->id . ']" value="' . esc_attr(isset($fixed[$method->id]) ? $fixed[$method->id] : '') . '" style="width: 50px;"> + <input type="number" step="any" min="0" name="rfmp_payment_method_variable[' . $method->id . ']" value="' . esc_attr(isset($variable[$method->id]) ? $variable[$method->id] : '') . '" style="width: 50px;"> %<br>';
                    echo '<hr>';
                }
            }


        } catch (Exception $e) {
            echo '<p style="color: red">' . $e->getMessage() . '</p>';
        }
    }

    /**
     * Save meta boxes
     *
     * @param $postId
     */
    public function saveMetaBoxes($postId)
    {
        // verify meta box nonce
        if (!isset($_POST['rfmp_meta_box_fields_nonce']) || !wp_verify_nonce($_POST['rfmp_meta_box_fields_nonce'], basename(__FILE__)))
            return;

        // verify meta box nonce
        if (!isset($_POST['rfmp_meta_box_priceoptions_nonce']) || !wp_verify_nonce($_POST['rfmp_meta_box_priceoptions_nonce'], basename(__FILE__)))
            return;

        // verify meta box nonce
        if (!isset($_POST['rfmp_meta_box_settings_nonce']) || !wp_verify_nonce($_POST['rfmp_meta_box_settings_nonce'], basename(__FILE__)))
            return;

        // verify meta box nonce
        if (!isset($_POST['rfmp_meta_box_paymentmethods_nonce']) || !wp_verify_nonce($_POST['rfmp_meta_box_paymentmethods_nonce'], basename(__FILE__)))
            return;

        // verify meta box nonce
        if (!isset($_POST['rfmp_meta_box_emails_nonce']) || !wp_verify_nonce($_POST['rfmp_meta_box_emails_nonce'], basename(__FILE__)))
            return;

        // Check the user's permissions.
        if (!current_user_can('edit_post', $postId))
            return;

        // Store custom fields
        update_post_meta($postId, '_rfmp_api_key', $_POST['rfmp_api_key']);
        update_post_meta($postId, '_rfmp_api_type', $_POST['rfmp_api_type']);
        update_post_meta($postId, '_rfmp_label_display', $_POST['rfmp_label_display']);
        update_post_meta($postId, '_rfmp_payment_methods_display', $_POST['rfmp_payment_methods_display']);
        update_post_meta($postId, '_rfmp_priceoptions_display', $_POST['rfmp_priceoptions_display']);
        update_post_meta($postId, '_rfmp_after_payment', $_POST['rfmp_after_payment']);
        update_post_meta($postId, '_rfmp_redirect_success', $_POST['rfmp_redirect_success']);
        update_post_meta($postId, '_rfmp_redirect_error', $_POST['rfmp_redirect_error']);
        update_post_meta($postId, '_rfmp_class_success', $_POST['rfmp_class_success']);
        update_post_meta($postId, '_rfmp_class_error', $_POST['rfmp_class_error']);
        update_post_meta($postId, '_rfmp_payment_description', $_POST['rfmp_payment_description']);
        update_post_meta($postId, '_rfmp_msg_success', $_POST['rfmp_msg_success']);
        update_post_meta($postId, '_rfmp_msg_error', $_POST['rfmp_msg_error']);
        update_post_meta($postId, '_rfmp_class_form', $_POST['rfmp_class_form']);
        update_post_meta($postId, '_rfmp_locale', $_POST['rfmp_locale']);
        update_post_meta($postId, '_rfmp_currency', $_POST['rfmp_currency']);
        update_post_meta($postId, '_rfmp_shipping_costs', $_POST['rfmp_shipping_costs']);
        update_post_meta($postId, '_rfmp_vat_setting', $_POST['rfmp_vat_setting']);

        // Disable non-supported payment methods if non-EUR
        if ($_POST['rfmp_currency'] != 'EUR')
        {
            $creditcard = false;
            $paypal = false;

            if (isset($_POST['rfmp_payment_method']['creditcard']) && $_POST['rfmp_payment_method']['creditcard'] == '1')
                $creditcard = true;

            if (isset($_POST['rfmp_payment_method']['paypal']) && $_POST['rfmp_payment_method']['paypal'] == '1')
                $paypal = true;

            foreach ($_POST['rfmp_payment_method'] as $method => $value)
            {
                $_POST['rfmp_payment_method'][$method] = '0';
            }

            if ($creditcard)
                $_POST['rfmp_payment_method']['creditcard'] = '1';

            if ($paypal)
                $_POST['rfmp_payment_method']['paypal'] = '1';
        }

        // Add address fields when API type is "orders"
        if ($_POST['rfmp_api_type'] == 'orders') {
            if (!in_array('address', $_POST['rfmp_fields_type'])) {
                $_POST['rfmp_fields_type']      = array_merge($_POST['rfmp_fields_type'], array('address'));
                $_POST['rfmp_fields_label']     = array_merge($_POST['rfmp_fields_label'], array(__('Address', 'mollie-forms')));
                $_POST['rfmp_fields_value']     = array_merge($_POST['rfmp_fields_value'], array(''));
                $_POST['rfmp_fields_class']     = array_merge($_POST['rfmp_fields_class'], array(''));
                $_POST['rfmp_fields_required']  = array_merge($_POST['rfmp_fields_required'], array(1));
            }
            if (!in_array('postalCode', $_POST['rfmp_fields_type'])) {
                $_POST['rfmp_fields_type']      = array_merge($_POST['rfmp_fields_type'], array('postalCode'));
                $_POST['rfmp_fields_label']     = array_merge($_POST['rfmp_fields_label'], array(__('Postal code', 'mollie-forms')));
                $_POST['rfmp_fields_value']     = array_merge($_POST['rfmp_fields_value'], array(''));
                $_POST['rfmp_fields_class']     = array_merge($_POST['rfmp_fields_class'], array(''));
                $_POST['rfmp_fields_required']  = array_merge($_POST['rfmp_fields_required'], array(1));
            }
            if (!in_array('city', $_POST['rfmp_fields_type'])) {
                $_POST['rfmp_fields_type']      = array_merge($_POST['rfmp_fields_type'], array('city'));
                $_POST['rfmp_fields_label']     = array_merge($_POST['rfmp_fields_label'], array(__('City', 'mollie-forms')));
                $_POST['rfmp_fields_value']     = array_merge($_POST['rfmp_fields_value'], array(''));
                $_POST['rfmp_fields_class']     = array_merge($_POST['rfmp_fields_class'], array(''));
                $_POST['rfmp_fields_required']  = array_merge($_POST['rfmp_fields_required'], array(1));
            }
            if (!in_array('country', $_POST['rfmp_fields_type'])) {
                $_POST['rfmp_fields_type']      = array_merge($_POST['rfmp_fields_type'], array('country'));
                $_POST['rfmp_fields_label']     = array_merge($_POST['rfmp_fields_label'], array(__('Country', 'mollie-forms')));
                $_POST['rfmp_fields_value']     = array_merge($_POST['rfmp_fields_value'], array(''));
                $_POST['rfmp_fields_class']     = array_merge($_POST['rfmp_fields_class'], array(''));
                $_POST['rfmp_fields_required']  = array_merge($_POST['rfmp_fields_required'], array(1));
            }
        }

        update_post_meta($postId, '_rfmp_fields_type', $_POST['rfmp_fields_type']);
        update_post_meta($postId, '_rfmp_fields_label', $_POST['rfmp_fields_label']);
        update_post_meta($postId, '_rfmp_fields_value', $_POST['rfmp_fields_value']);
        update_post_meta($postId, '_rfmp_fields_class', $_POST['rfmp_fields_class']);
        update_post_meta($postId, '_rfmp_fields_required', $_POST['rfmp_fields_required']);

        update_post_meta($postId, '_rfmp_payment_method', $_POST['rfmp_payment_method']);
        update_post_meta($postId, '_rfmp_payment_method_fixed', $_POST['rfmp_payment_method_fixed']);
        update_post_meta($postId, '_rfmp_payment_method_variable', $_POST['rfmp_payment_method_variable']);

        update_post_meta($postId, '_rfmp_enabled_paid_customer', $_POST['rfmp_enabled_paid_customer']);
        update_post_meta($postId, '_rfmp_email_paid_customer', $_POST['rfmp_email_paid_customer']);
        update_post_meta($postId, '_rfmp_subject_paid_customer', $_POST['rfmp_subject_paid_customer']);
        update_post_meta($postId, '_rfmp_fromname_paid_customer', $_POST['rfmp_fromname_paid_customer']);
        update_post_meta($postId, '_rfmp_fromemail_paid_customer', $_POST['rfmp_fromemail_paid_customer']);
        update_post_meta($postId, '_rfmp_enabled_expired_customer', $_POST['rfmp_enabled_expired_customer']);
        update_post_meta($postId, '_rfmp_email_expired_customer', $_POST['rfmp_email_expired_customer']);
        update_post_meta($postId, '_rfmp_subject_expired_customer', $_POST['rfmp_subject_expired_customer']);
        update_post_meta($postId, '_rfmp_fromname_expired_customer', $_POST['rfmp_fromname_expired_customer']);
        update_post_meta($postId, '_rfmp_fromemail_expired_customer', $_POST['rfmp_fromemail_expired_customer']);
        update_post_meta($postId, '_rfmp_enabled_cancelled_customer', $_POST['rfmp_enabled_cancelled_customer']);
        update_post_meta($postId, '_rfmp_email_cancelled_customer', $_POST['rfmp_email_cancelled_customer']);
        update_post_meta($postId, '_rfmp_subject_cancelled_customer', $_POST['rfmp_subject_cancelled_customer']);
        update_post_meta($postId, '_rfmp_fromname_cancelled_customer', $_POST['rfmp_fromname_cancelled_customer']);
        update_post_meta($postId, '_rfmp_fromemail_cancelled_customer', $_POST['rfmp_fromemail_cancelled_customer']);

        update_post_meta($postId, '_rfmp_enabled_paid_merchant', $_POST['rfmp_enabled_paid_merchant']);
        update_post_meta($postId, '_rfmp_email_paid_merchant', $_POST['rfmp_email_paid_merchant']);
        update_post_meta($postId, '_rfmp_subject_paid_merchant', $_POST['rfmp_subject_paid_merchant']);
        update_post_meta($postId, '_rfmp_fromname_paid_merchant', $_POST['rfmp_fromname_paid_merchant']);
        update_post_meta($postId, '_rfmp_fromemail_paid_merchant', $_POST['rfmp_fromemail_paid_merchant']);
        update_post_meta($postId, '_rfmp_toemail_paid_merchant', $_POST['rfmp_toemail_paid_merchant']);
        update_post_meta($postId, '_rfmp_enabled_expired_merchant', $_POST['rfmp_enabled_expired_merchant']);
        update_post_meta($postId, '_rfmp_email_expired_merchant', $_POST['rfmp_email_expired_merchant']);
        update_post_meta($postId, '_rfmp_subject_expired_merchant', $_POST['rfmp_subject_expired_merchant']);
        update_post_meta($postId, '_rfmp_fromname_expired_merchant', $_POST['rfmp_fromname_expired_merchant']);
        update_post_meta($postId, '_rfmp_fromemail_expired_merchant', $_POST['rfmp_fromemail_expired_merchant']);
        update_post_meta($postId, '_rfmp_toemail_expired_merchant', $_POST['rfmp_toemail_expired_merchant']);
        update_post_meta($postId, '_rfmp_enabled_cancelled_merchant', $_POST['rfmp_enabled_cancelled_merchant']);
        update_post_meta($postId, '_rfmp_email_cancelled_merchant', $_POST['rfmp_email_cancelled_merchant']);
        update_post_meta($postId, '_rfmp_subject_cancelled_merchant', $_POST['rfmp_subject_cancelled_merchant']);
        update_post_meta($postId, '_rfmp_fromname_cancelled_merchant', $_POST['rfmp_fromname_cancelled_merchant']);
        update_post_meta($postId, '_rfmp_fromemail_cancelled_merchant', $_POST['rfmp_fromemail_cancelled_merchant']);
        update_post_meta($postId, '_rfmp_toemail_cancelled_merchant', $_POST['rfmp_toemail_cancelled_merchant']);

        $sortOrder = 0;
        foreach ($_POST['rfmp_priceoptions_new'] as $key => $new)
        {
            if ($new == '1')
            {
                // new row
                $this->db->insert($this->mollieForms->getPriceOptionsTable(), array(
                    'post_id'           => $postId,
                    'description'       => $_POST['rfmp_priceoptions_desc'][$key],
                    'price'             => $_POST['rfmp_priceoptions_price'][$key] ?: null,
                    'price_type'        => $_POST['rfmp_priceoptions_pricetype'][$key],
                    'vat'               => $_POST['rfmp_priceoptions_vat'][$key] ?: null,
                    'frequency'         => $_POST['rfmp_priceoptions_frequency'][$key],
                    'frequency_value'   => $_POST['rfmp_priceoptions_frequencyval'][$key] ?: null,
                    'times'             => $_POST['rfmp_priceoptions_times'][$key] ?: null,
                    'stock'             => $_POST['rfmp_priceoptions_stock'][$key] != '' ? $_POST['rfmp_priceoptions_stock'][$key] : null,
                    'sort_order'        => $sortOrder
                ));
            }
            elseif ($new == '-1')
            {
                if (strstr($key, 'po-')) {
                    // delete row
                    $this->db->delete($this->mollieForms->getPriceOptionsTable(), array(
                        'ID' => str_replace('po-', '', $key)
                    ));
                }
            }
            else
            {
                // existing row
                $this->db->update($this->mollieForms->getPriceOptionsTable(), array(
                    'description'       => $_POST['rfmp_priceoptions_desc'][$key],
                    'price'             => $_POST['rfmp_priceoptions_price'][$key] ?: null,
                    'price_type'        => $_POST['rfmp_priceoptions_pricetype'][$key],
                    'vat'               => $_POST['rfmp_priceoptions_vat'][$key] ?: null,
                    'frequency'         => $_POST['rfmp_priceoptions_frequency'][$key],
                    'frequency_value'   => $_POST['rfmp_priceoptions_frequencyval'][$key] ?: null,
                    'times'             => $_POST['rfmp_priceoptions_times'][$key] ?: null,
                    'stock'             => $_POST['rfmp_priceoptions_stock'][$key] != '' ? $_POST['rfmp_priceoptions_stock'][$key] : null,
                    'sort_order'        => $sortOrder
                ),
                array(
                    'ID' => str_replace('po-', '', $key)
                ));
            }

            $sortOrder++;
        }
    }

    /**
     * Registrations page
     */
    public function pageRegistrations()
    {
        $table = new RegistrationsTable($this->mollieForms);
        $table->prepare_items();

        if (isset($_GET['post']))
            $post = get_post($_GET['post']);

        if (isset($_GET['msg']))
        {
            switch ($_GET['msg'])
            {
                case 'delete-ok':
                    $msg = '<div class="updated notice"><p>' . esc_html__('The registration is successful deleted', 'mollie-forms') . '</p></div>';
                    break;
            }

            echo isset($msg) ? $msg : '';
        }
        ?>
        <div class="wrap">
            <h2><?php esc_html_e('Registrations', 'mollie-forms'); echo (isset($post) ? ' <small>(' . $post->post_title . ')</small>' : '');?></h2>

            <?php $table->display();?>
        </div>
        <?php
    }

    /**
     * Registration page
     *
     * @return mixed
     * @throws Exception
     */
    public function pageRegistration()
    {
        if (!isset($_GET['view']))
            return esc_html__('Registration not found', 'mollie-forms');

        $id = (int) $_GET['view'];

        $registration   = $this->db->get_row("SELECT * FROM {$this->mollieForms->getRegistrationsTable()} WHERE id=" . $id);
        if ($registration == null)
            return esc_html__('Registration not found', 'mollie-forms');

        // Delete registration
        if (isset($_GET['delete']) && check_admin_referer('delete-reg_' . $_GET['view']))
        {
            $this->db->delete($this->mollieForms->getRegistrationsTable(), array(
                'id' => $id
            ));

            wp_redirect('?post_type=mollie-forms&page=registrations&msg=delete-ok');
            exit;
        }

        // subscriptions table fix
        if ($registration->subs_fix)
            $subsTable = $this->mollieForms->getSubscriptionsTable();
        else
            $subsTable = $this->mollieForms->getCustomersTable();

        // Connect with Mollie
        $apiKey = get_post_meta($registration->post_id, '_rfmp_api_key', true);
        $mollie = new MollieApi($apiKey);

        $vatSetting = get_post_meta($registration->post_id, '_rfmp_vat_setting', true);

        // Get all subscriptions
        $allSubs = $mollie->all('customers/' . $registration->customer_id . '/subscriptions');
        foreach ($allSubs as $sub)
        {
            $this->db->update($subsTable, array(
                'sub_status' => $sub->status
            ), array(
                'subscription_id' => $sub->id
            ));
        }

        $fields         = $this->db->get_results("SELECT * FROM {$this->mollieForms->getRegistrationFieldsTable()} WHERE registration_id=" . $id);
        $subscriptions  = $this->db->get_results("SELECT * FROM {$subsTable} WHERE registration_id=" . $id);
        $payments       = $this->db->get_results("SELECT * FROM {$this->mollieForms->getPaymentsTable()} WHERE registration_id=" . $id);
        $priceOptions   = $this->db->get_results("SELECT * FROM {$this->mollieForms->getRegistrationPriceOptionsTable()} WHERE registration_id=" . $id);

        // Cancel subscription
        if (isset($_GET['cancel']) && check_admin_referer('cancel-sub_' . $_GET['cancel']))
        {
            try {
                $mollie->delete('customers/' . $registration->customer_id . '/subscriptions/' . $_GET['cancel']);
                $this->db->update($subsTable, array(
                    'sub_status' => 'cancelled'
                ), array(
                    'subscription_id' => $_GET['cancel']
                ));

                wp_redirect('?post_type=' . $_REQUEST['post_type'] . '&page=' . $_REQUEST['page'] . '&view=' . $_REQUEST['view'] . '&msg=cancel-ok');
            } catch(Exception $e) {
                echo '<div class="error notice">' . $e->getMessage() . '</div>';
            }
        }

        // Refund payment
        if (isset($_GET['refund']) && check_admin_referer('refund-payment_' . $_GET['refund']))
        {
            try {
                if (substr($_GET['refund'], 0, 3) == 'ord') {
                    $mollie->post('orders/' . $_GET['refund'] . '/refunds', array(
                        'lines' => array()
                    ));
                } else {
                    $payment = $mollie->get('payments/' . $_GET['refund']);
                    $mollie->post('payments/' . $payment->id . '/refunds', array(
                        'amount' => array(
                            'currency'  => $payment->amount->currency,
                            'value'     => $payment->amount->value
                        )
                    ));
                }

                $this->db->update($this->mollieForms->getPaymentsTable(), array(
                    'payment_status' => 'refunded'
                ), array(
                    'payment_id' => $_GET['refund']
                ));

                wp_redirect('?post_type=' . $_REQUEST['post_type'] . '&page=' . $_REQUEST['page'] . '&view=' . $_REQUEST['view'] . '&msg=refund-ok');

            } catch(Exception $e) {
                wp_redirect('?post_type=' . $_REQUEST['post_type'] . '&page=' . $_REQUEST['page'] . '&view=' . $_REQUEST['view'] . '&msg=refund-nok');
            }
        }

        if (isset($_GET['msg']))
        {
            switch ($_GET['msg'])
            {
                case 'refund-ok':
                    $msg = '<div class="updated notice"><p>' . esc_html__('The payment is successful refunded', 'mollie-forms') . '</p></div>';
                    break;
                case 'refund-nok':
                    $msg = '<div class="error notice"><p>' . esc_html__('The payment can not be refunded', 'mollie-forms') . '</p></div>';
                    break;
                case 'cancel-ok':
                    $msg = '<div class="updated notice"><p>' . esc_html__('The subscription is successful cancelled', 'mollie-forms') . '</p></div>';
                    break;
            }

            echo isset($msg) ? $msg : '';
        }

        include $this->mollieForms->getDirPath() . 'templates/admin/registration.php';
    }

    /**
     * Addons page
     */
    public function pageAddons()
    {
        ?>
        <div class="wrap">
            <h2><?php _e('Add-ons', 'mollie-forms');?></h2>

            <ul class="products">
                <li class="product">
                    <a href="https://wobbie.nl/downloads/mailchimp-for-mollie-forms/" target="_blank">
                        <h2><?php _e('Mailchimp', 'mollie-forms');?></h2>
                        <p><?php _e('Add people to your Mailchimp mailing list.', 'mollie-forms');?></p>
                    </a>
                </li>
            </ul>
        </div>

        <?php
    }

    /**
     * Generate export of registrations
     */
    public function exportRegistrations()
    {
        $postId = $_GET['post'];

        header('Content-Type: text/csv; charset=utf-8');
        header('Content-Disposition: attachment; filename=registrations.csv');
        $output = fopen('php://output', 'w');

        $headers = array(
            esc_html__('Status', 'mollie-forms'),
            esc_html__('Date/time', 'mollie-forms'),
            esc_html__('Currency', 'mollie-forms'),
            esc_html__('Total', 'mollie-forms'),
            esc_html__('Description', 'mollie-forms'),
            esc_html__('Price options', 'mollie-forms'),
        );

        // get all fields for headers
        $registration = $this->db->get_row("SELECT * FROM {$this->mollieForms->getRegistrationsTable()} WHERE post_id=" . (int) $postId . " ORDER BY id DESC LIMIT 1");
        $fields = $this->db->get_results("SELECT * FROM {$this->mollieForms->getRegistrationFieldsTable()} WHERE registration_id=" . (int) $registration->id);
        foreach ($fields as $field)
            $headers[] = esc_html($field->field);

        // put header in csv
        fputcsv($output, $headers);

        // make all registration rows
        $registrations = $this->db->get_results("SELECT * FROM {$this->mollieForms->getRegistrationsTable()} WHERE post_id=" . (int) $postId . " ORDER BY id DESC");
        foreach ($registrations as $registration)
        {
            $paymentsPaid = $this->db->get_var("SELECT COUNT(*) FROM {$this->mollieForms->getPaymentsTable()} WHERE payment_status='paid' AND registration_id=" . (int) $registration->id);

            $priceOptions = $this->db->get_results("SELECT * FROM {$this->mollieForms->getRegistrationPriceOptionsTable()} WHERE registration_id=" . (int) $registration->id);
            $options = array();
            foreach ($priceOptions as $priceOption) {
                $currency   = $priceOption->currency ?: 'EUR';
                $frequency  = $priceOption->frequency_value . ' ' . $priceOption->frequency;
                $options[]  = $priceOption->quantity . 'x ' . $priceOption->description . ' (' . ($priceOption->currency ?: 'EUR') . ' ' . number_format($priceOption->price, $this->helpers->getCurrencies($currency), '.', '') . ' ' . $this->helpers->getFrequencyLabel($frequency) . ')';
            }

            $rows = array(
                $paymentsPaid ? __('Paid', 'mollie-forms') : __('Not paid', 'mollie-forms'),
                $registration->created_at,
                $registration->currency ?: 'EUR',
                $registration->total_price,
                $registration->description,
                implode(' | ', $options)
            );

            $fields = $this->db->get_results("SELECT * FROM {$this->mollieForms->getRegistrationFieldsTable()} WHERE registration_id=" . (int) $registration->id);
            foreach ($fields as $field)
                $rows[] = esc_html($field->value);

            // put row in csv
            fputcsv($output, $rows);
        }
    }
}